package ucot.heuristic;


import java.util.Properties;
import java.util.Set;

import ucot.model.AnalyzeModel;
import ucot.model.Entity;
import ucot.model.Method;
import ucot.model.Relation;
import ucot.model.RelationType;
import ucot.parser.ParsedData;
import ucot.parser.Sentence;
import ucot.parser.Word;
/**
 * Abbotts Heuristic is a way to etsimated which
 * parts of the speech are relevant and in which
 * way when doing object analyze.
 * 
 * @author pajumasu
 *
 */
public class AbbottsHeuristic implements HeuristicInterface {
	private Properties options = new Properties();
    

	
	
	/**
	 * Searches analyzemodel for a entity with given name. If none is found, returns a new one
	 * @param name name of the entity to look for
	 * @param model analyzemodel to look from
	 * @return entity with given name
	 */
	private Entity getEntityCalled(String name, AnalyzeModel model){		
		Entity ent = model.getEntityByName(name);
		if (null == ent){
			ent = new Entity(name);			
		}
		return ent;
	}
	
	
	/** 
	 * @see ucot.heuristic.HeuristicInterface#doHeuristic(ucot.parser.ParsedData, ucot.model.AnalyzeModel)
	 */
    public void doHeuristic(ParsedData data, AnalyzeModel model) throws Exception {
		if (model == null || data == null) 
			return;

        
        //Map<String, Entity> nameToEntity = new HashMap<String,Entity>();
        
        
        // Loop through all of the sences.
        for (Sentence s : data.getSentences()) {
            
            // Loop through all the words.
            for (Word word : s.getWords()) {
                
                // Basic information from the word.
                String basicForm = word.getBasicForm();
                String wordClass = word.getWordClass();
                
                if (wordClass.equals("noun")) {
                    // Word is noun. So its is probably an entity.
                    model.addItem(getEntityCalled(basicForm,model));
                } else if (wordClass.equals("verb")) {
                    // Word is verb. So it is probably a operation
                    // between entities.
                    
                    // Check if there exist information 
                    // about the structure of the sentence. 
                    Word subject = word.getLinked("subject");
                    Word object = word.getLinked("object");
                    
                    if (null != subject && null != object) {
                        // There is structural information
                        // so we can create entity to entity
                        // relation.
                        
                        Entity subjectE = getEntityCalled(subject.getBasicForm(), model);
                        Entity objectE = getEntityCalled(object.getBasicForm(), model);
                        
                        // Create relation called by the name of the verb.
                        Relation relation = new Relation(subjectE, word
                                .getBasicForm(), objectE);
                        
                        model.addRelation(relation);
                    }
                    
                    if (null != subject){
                        // We know the subject
                        // so we can add "method" to the entity.
                        
                        Method method = null;
                        
                        
                        Entity subjectE = getEntityCalled(subject.getBasicForm(), model);
                        
                        
                        // Code blocks finds equally named methods for the
                        // entity.
                        Set<Relation> rels = model.getRelationsFrom(subjectE,"method");
                        for(Relation relation : rels) {
                            if (relation.getTo().getName().equals(word.getBasicForm())){
                                method = (Method) relation.getTo();
                                break;
                            }
                        }
                        
                        
                        if (null == method)  {
                            // No method was found so create one.
                            // If methdo was found then there is no
                            // need to add anything.
                            method = new Method(word.getBasicForm());
                            Relation relation = new Relation(subjectE, RelationType.CONTAINS, method);
                            model.addRelation(relation);
                        }                        
                    }
                }
            }
        }        
    }

	
	public String getName() {		
		return "Abbots heuristic";
	}

	public void setOptions(Properties options) {
		this.options = options;		
	}

	public Properties getOptions() {
		return options;
	}    
}
